# -*- coding: utf-8 -*-
###############################################################################
# Copyright (c) 2014-2019 Mixlinker Networks Inc. <mixiot@mixlinker.com>
# All rights reserved.
#
# This program and the accompanying materials are made available under the
# terms of the Application License of Mixlinker Networks License and Mixlinker
# Distribution License which accompany this distribution.
#
# The Mixlinker License is available at
#    http://www.mixlinker.com/legal/license.html
# and the Mixlinker Distribution License is available at
#    http://www.mixlinker.com/legal/distribution.html
#
# Contributors:
#    Mixlinker Technical Team
###############################################################################
# Description : 数据传输
# Author : kanxiangyang

import ipywidgets as widgets
from ipywidgets import interact_manual
from IPython.core.interactiveshell import InteractiveShell
import pandas as pd
import boto3
import tarfile
import os
import mavis
from . import config

# 交互设置
interact_manual.opts['manual_name'] = '确认'
InteractiveShell.ast_node_interactivity = 'all'


def _set_config(aws_access_key_id, aws_secret_access_key):
    if not os.path.isdir('/root/.aws'):
        os.mkdir('/root/.aws')
    credentials = open('/root/.aws/credentials', 'w+')
    credentials.write(
        f"[default]\naws_access_key_id = {aws_access_key_id}\naws_secret_access_key = {aws_secret_access_key}")
    aws_config = open('/root/.aws/config', 'w+')
    aws_config.write(f"[default]\nregion = cn-northwest-1\noutput = json")
    print('AWS S3数据桶凭据设置成功')


def set_config():
    """
    设置AWS S3数据桶传输凭据

    :param str aws_access_key_id: 数据桶访问密钥ID
    :param str aws_secret_access_key: 数据桶访问密钥

    .. note::
        数据桶访问ID和访问密钥可以登录AWS账户获取，默认地区为cn-northwest-1

        https://console.amazonaws.cn/iam/home?region=cn-northwest-1#/security_credentials

        服务器中的设置保存在/root/.aws/config和/root/.aws/credentials中

    .. jupyter-execute::
        :hide-code:

        import ipywidgets as widgets
        from ipywidgets import interact_manual
        interact_manual.opts['manual_name'] = '确认'

        def set_config_demo(aws_access_key_id, aws_secret_access_key):
            print("演示成功")

        interact_manual(set_config_demo,
                        aws_access_key_id=widgets.Text(value='请输入访问密钥 ID', description='访问密钥ID', disabled=False),
                        aws_secret_access_key=widgets.Text(value='请输入访问密钥', description='访问密钥', disabled=False));
    """
    print("AWS S3数据桶安全凭证")
    print("https://console.amazonaws.cn/iam/home?region=cn-northwest-1#/security_credentials")
    return interact_manual(_set_config,
                           aws_access_key_id=widgets.Text(value='请输入访问密钥 ID', description='访问密钥ID', disabled=False),
                           aws_secret_access_key=widgets.Text(value='请输入访问密钥', description='访问密钥', disabled=False)
                           )


def _upload(filename, directory, compress=True):
    client = boto3.client('s3')
    if compress:
        tarfile_name = filename.split(".", 1)[0] + ".tar.gz"
        t = tarfile.open(tarfile_name, "w:gz")
        t.add(filename)
        t.close()
        print('压缩成功')
        tarfile_name = tarfile_name.split("/")[-1]
        client.upload_file(Filename=tarfile_name, Bucket=config.bucket_name,
                           Key=directory + "/" + tarfile_name)
        print('上传成功')
        os.remove(tarfile_name)
        print(f'保存路径:\n目录:{directory}\n文件:{tarfile_name}')
    else:
        client.upload_file(Filename=filename, Bucket=config.bucket_name, Key=directory + "/" + filename.split("/")[-1])
        filename = filename.split("/")[-1]
        print('上传成功')
        print(f'保存路径:\n目录:{directory}\n文件:{filename}')


def upload():
    """
    数据上传至AWS S3数据桶

    :param str filename: 要上传到数据桶的本地文件路径
    :param str directory: 上传到数据桶中的该目录下
    :param bool compress: 是否压缩文件，压缩格式为tar.gz

    .. note::
        本地文件路径可以是文件的绝对路径或者相对路径，上传目录不需要打斜杠'/'

    .. jupyter-execute::
        :hide-code:

        import ipywidgets as widgets
        from ipywidgets import interact_manual
        interact_manual.opts['manual_name'] = '确认'

        def upload_demo(filename, directory, compress=True):
            print()

        interact_manual(upload_demo,
                        filename=widgets.Text(value='请输入本地文件路径', description='本地文件名', disabled=False),
                        directory=widgets.Text(value='请指定上传文件所在文件夹名', description='上传目录', disabled=False),
                        compress=widgets.Checkbox(value=True, description='是否压缩', disabled=False));
    """
    return interact_manual(_upload,
                           filename=widgets.Text(value='请输入本地文件路径', description='本地文件名', disabled=False),
                           directory=widgets.Text(value='请指定上传文件所在文件夹名', description='上传目录', disabled=False),
                           compress=widgets.Checkbox(value=True, description='是否压缩', disabled=False))


def _download(directory, key, filename):
    filedir = os.path.dirname(filename)
    if not os.path.isdir(filedir) and filedir:
        os.makedirs(filedir, mode=0o777)
    boto3.client('s3').download_file(Bucket=config.bucket_name, Key=directory + "/" + key, Filename=filename)
    print('下载成功')
    if not filedir:
        filename = './' + filename
    print(f'保存路径为{filename}')


def download():
    """
    从AWS S3数据桶下载数据

    :param str directory: 下载目录
    :param str key: 下载文件名
    :param str filename: 保存文件名

    .. jupyter-execute::
        :hide-code:

        import ipywidgets as widgets
        from ipywidgets import interact_manual
        interact_manual.opts['manual_name'] = '确认'

        def download_demo(directory, key, filename):
            print()

        interact_manual(download_demo,
                        directory=widgets.Text(value='请输入下载文件所在文件夹名', description='下载目录', disabled=False),
                        key=widgets.Text(value='请输入下载文件名称', description='下载文件名', disabled=False),
                        filename=widgets.Text(value='请输入保存文件路径', description='保存文件名', disabled=False));

    """
    return interact_manual(_download,
                           directory=widgets.Text(value='请输入下载文件所在文件夹名', description='下载目录', disabled=False),
                           key=widgets.Text(value='请输入下载文件名称', description='下载文件名', disabled=False),
                           filename=widgets.Text(value='请输入保存文件路径', description='保存文件名', disabled=False))


def read(directory, key):
    """
    读取AWS S3数据桶上的数据

    :param str directory: 数据桶目录
    :param str key: 数据桶文件名
    :return: pandas.DataFrame

    """
    prefix = "/data/tmp/"
    filename = prefix + directory + "/" + key
    if not os.path.isdir(prefix + directory):
        os.makedirs(prefix + directory, mode=0o777)
    if not os.path.exists(filename):
        boto3.client("s3").download_file(Bucket=config.bucket_name, Key=directory + "/" + key, Filename=filename)
    if key.endswith("tar.gz"):
        t = tarfile.open(filename)
        t.extract(prefix + directory)
        os.remove(filename)
        filename = filename.rstrip("tar.gz")
    else:
        pass
    if filename.endswith('.csv'):
        data = pd.read_csv(filename, index_col=0, parse_dates=True).sort_index(axis=0, ascending=True)
    elif filename.endswith('.h5'):
        data = pd.read_hdf(filename).sort_index(axis=0, ascending=True)
    else:
        raise IOError('路径格式错误，请输入csv格式或h5格式')
    return data




# -*- coding: utf-8 -*-
###############################################################################
# Copyright (c) 2014-2019 Mixlinker Networks Inc. <mixiot@mixlinker.com>
# All rights reserved.
#
# This program and the accompanying materials are made available under the
# terms of the Application License of Mixlinker Networks License and Mixlinker
# Distribution License which accompany this distribution.
#
# The Mixlinker License is available at
#    http://www.mixlinker.com/legal/license.html
# and the Mixlinker Distribution License is available at
#    http://www.mixlinker.com/legal/distribution.html
#
# Contributors:
#    Mixlinker Technical Team
###############################################################################
# Description : 固定化的参数
# Author : zhaoguangjun
class Config():
    ## train and test
    model_path = "best_model_i5_o1-S47.pkl"
    batch_size = 64
    epochs = 200    
    optimizers = "sgd"
    learning_rate = 1e-5
    beta1 = 0.5
    idx = 1092990

    ## data
    map_name = "./DEVICE1017_map.csv "  
    train_file = "./train_SK.csv"    
    test_file = "./test_SK.csv"
    max_read_len = 900000
    in_keys = ["SK", "S12", "S09", "S33", "S21"]
    out_keys = ["S47"]
    
    ## model
    in_class = len(in_keys)
    out_class = len(out_keys)
    hidden_dim = 2048






from .core import Analysis
from .plot import MavisPlot, set_figsize
from .transfer import (
    set_config,
    upload,
    download,
    read
)
from .api import (
    Mixiot,
    Export,
    Mapping,
    get_token,
    get_menu_list,
    get_mapping,
    get_data,
    export_data
)
from . import __info__
from . import config
from .model.api import (
    ModelDemo,
    IForest,
    Boiler
)


# -*- coding: utf-8 -*-
###############################################################################
# Copyright (c) 2014-2019 Mixlinker Networks Inc. <mixiot@mixlinker.com>
# All rights reserved.
#
# This program and the accompanying materials are made available under the
# terms of the Application License of Mixlinker Networks License and Mixlinker
# Distribution License which accompany this distribution.
#
# The Mixlinker License is available at
#    http://www.mixlinker.com/legal/license.html
# and the Mixlinker Distribution License is available at
#    http://www.mixlinker.com/legal/distribution.html
#
# Contributors:
#    Mixlinker Technical Team
###############################################################################
# Description : 数据可视化
# Author : kanxiangyang

import pandas as pd
from scipy import stats
import ipywidgets as widgets
from ipywidgets import interact_manual
import matplotlib as mpl
import matplotlib.pyplot as plt
import seaborn as sns
from IPython.core.interactiveshell import InteractiveShell
from IPython.display import display, FileLink
from .api import create_download_link
import os

# matplotlib中文化支持
mpl.rcParams['font.sans-serif'] = ['simhei']
mpl.rcParams['font.family'] = 'sans-serif'
mpl.rcParams['axes.unicode_minus'] = False

# 展示的最大行数
pd.options.display.max_rows = 10

# 展示的最大列数
pd.options.display.max_columns = 10

# 交互设置
interact_manual.opts['manual_name'] = '确认'
InteractiveShell.ast_node_interactivity = 'all'


def set_figsize(length, width):
    """
    设置数据展示图大小

    :param str length: 图的长度
    :param str width: 图的宽度
    """
    mpl.rcParams["figure.figsize"] = [length, width]


class MavisPlot(pd.DataFrame):
    """
    数据预览与交互式可视化

    展示数据的基本统计量, 缺失值分析, 单参数趋势图, 单参数分布图, 单参数PP图，双参数散点图, 多参数关系图，多参数相关性热点图等信息

    :param data: 数据所在本地路径或DataFrame
    """

    def __init__(self, data):
        if isinstance(data, str):
            if data.endswith('.csv'):
                data = pd.read_csv(data, index_col=0, parse_dates=True).sort_index(axis=0, ascending=True)
            elif data.endswith('.h5'):
                data = pd.read_hdf(data).sort_index(axis=0, ascending=True)
            else:
                raise IOError('路径格式错误，请输入csv格式或h5格式')
        else:
            pass
        super().__init__(data=data)
        self.index = pd.to_datetime(self.index)

    def summary(self):
        """
        数据基本统计量, 包括

        样本数

        平均值

        标准差

        最小值

        1/4位数

        中位数

        3/4位数

        最大值
        """
        preview_df = self.describe().T
        preview_df.columns = ['样本数', '平均值', '标准差', '最小值', '1/4位数', '中位数', '3/4位数', '最大值']
        preview_df['偏度'] = self.skew()
        preview_df['峰度'] = self.kurt()
        filename = "data/preview_statistics.csv"
        if not os.path.exists("data/"):
            os.makedirs("data/")
        preview_df.to_csv(filename, encoding='utf_8_sig')
        print(f'参数基本统计量表:\n')
        # categorical_columns = self.columns[self.apply(lambda x: x.value_counts().shape[0] < 3, axis=0)].tolist()
        # print(f'0-1型状态参数：\n{categorical_columns}')
        display(FileLink(filename))
        return create_download_link(preview_df, title="下载到本地", filename=filename)

    def _naplot(self, columns, start_date, end_date, save=False):
        start_date = pd.to_datetime(start_date)
        end_date = pd.to_datetime(end_date)
        naplot_df = self.loc[(self.index >= start_date) & (self.index <= end_date)][list(columns)]
        percent_col = naplot_df.isnull().sum().sort_values(ascending=False) / naplot_df.shape[0]
        plt.subplots(figsize=(12, 6))
        plt.xticks(rotation='90')
        sns.barplot(x=list(columns), y=percent_col)
        plt.xlabel('参数', fontsize=15)
        plt.ylabel('缺失值比例', fontsize=15)
        plt.title('各参数的缺失值比例', fontsize=15)
        if save:
            plt.savefig("参数缺失情况.png", dpi=600)

    def naplot(self):
        """
        数据缺失值比例图

        :param list columns: 分析的参数列表
        :param str start_date: 开始日期
        :param str end_date: 结束日期
        :param bool save: 是否保存作图

        .. jupyter-execute::
            :hide-code:

            import ipywidgets as widgets
            from ipywidgets import interact_manual
            from datetime import datetime, timedelta
            interact_manual.opts['manual_name'] = '确认'

            columns_list = ['param_1', 'param_2', 'param_3', 'param_4']
            today = datetime.now()
            yesterday = datetime.now() - timedelta(1)

            def naplot_demo(columns, start_date, end_date, save=False):
                print('demo')

            interact_manual(naplot_demo,
                            columns=widgets.SelectMultiple(options=columns_list, value=columns_list[0:2],
                                                           description='多参数选择'),
                            start_date=widgets.DatePicker(value=yesterday, description='开始日期: '),
                            end_date=widgets.DatePicker(value=today, description='结束日期: '),
                            save=widgets.Checkbox(value=False, description='是否保存图像', disabled=False));
        """
        print('\n            数据缺失值比例图')
        return interact_manual(self._naplot,
                               columns=widgets.SelectMultiple(options=self.columns.tolist(),
                                                              value=self.columns[0:2].tolist(),
                                                              description='多参数选择'),
                               start_date=widgets.DatePicker(value=self.index[0], description='开始日期: '),
                               end_date=widgets.DatePicker(value=self.index[-1], description='结束日期: '),
                               save=widgets.Checkbox(value=False, description='是否保存图像', disabled=False))

    def _timeplot(self, column, start_date, end_date, save=False):
        start_date = pd.to_datetime(start_date)
        end_date = pd.to_datetime(end_date)
        timeplot_df = self.loc[(self.index >= start_date) & (self.index <= end_date)][column]
        set_figsize(10, 5)
        timeplot_df.plot()
        if save:
            plt.savefig(f"参数{column}的趋势图.png", dpi=600)

    def timeplot(self):
        """
        单参数趋势图

        :param str column: 分析参数
        :param str start_date: 开始日期
        :param str end_date: 结束日期
        :param bool save: 是否保存作图

        .. jupyter-execute::
            :hide-code:

            import ipywidgets as widgets
            from ipywidgets import interact_manual
            from datetime import datetime, timedelta
            interact_manual.opts['manual_name'] = '确认'

            columns = ['param_1', 'param_2', 'param_3', 'param_4']
            today = datetime.now()
            yesterday = datetime.now() - timedelta(1)

            def timeplot_demo(column, start_date, end_date, save=False):
                print()

            interact_manual(timeplot_demo,
                            column=widgets.Dropdown(options=columns, value=columns[0], description='参数: '),
                            start_date=widgets.DatePicker(value=yesterday, description='开始日期: '),
                            end_date=widgets.DatePicker(value=today, description='结束日期: '),
                            save=widgets.Checkbox(value=False, description='是否保存图像', disabled=False));
        """
        print('\n            单参数趋势图')
        return interact_manual(self._timeplot,
                               column=widgets.Dropdown(options=list(self.columns), value=self.columns[0],
                                                       description='参数: '),
                               start_date=widgets.DatePicker(value=self.index[0], description='开始日期: '),
                               end_date=widgets.DatePicker(value=self.index[-1], description='结束日期: '),
                               save=widgets.Checkbox(value=False, description='是否保存图像', disabled=False))

    def _distplot(self, column, start_date, end_date, save=False):
        start_date = pd.to_datetime(start_date)
        end_date = pd.to_datetime(end_date)
        distplot_df = self.loc[(self.index >= start_date) & (self.index <= end_date)][column]
        set_figsize(10, 5)
        sns.distplot(distplot_df)
        if save:
            plt.savefig(f"参数{column}的分布图.png", dpi=600)

    def distplot(self):
        """
        单参数分布图

        :param str column: 分析参数
        :param str start_date: 开始日期
        :param str end_date: 结束日期
        :param bool save: 是否保存作图

        .. jupyter-execute::
            :hide-code:

            import ipywidgets as widgets
            from ipywidgets import interact_manual
            from datetime import datetime, timedelta
            interact_manual.opts['manual_name'] = '确认'

            columns = ['param_1', 'param_2', 'param_3', 'param_4']
            today = datetime.now()
            yesterday = datetime.now() - timedelta(1)

            def distplot_demo(column, start_date, end_date, save=False):
                print()

            interact_manual(distplot_demo,
                            column=widgets.Dropdown(options=columns, value=columns[0], description='参数: '),
                            start_date=widgets.DatePicker(value=yesterday, description='开始日期: '),
                            end_date=widgets.DatePicker(value=today, description='结束日期: '),
                            save=widgets.Checkbox(value=False, description='是否保存图像', disabled=False));
        """
        print('\n            单参数分布图')
        return interact_manual(self._distplot,
                               column=widgets.Dropdown(options=list(self.columns), value=self.columns[0],
                                                       description='参数: '),
                               start_date=widgets.DatePicker(value=self.index[0], description='开始日期: '),
                               end_date=widgets.DatePicker(value=self.index[-1], description='结束日期: '),
                               save=widgets.Checkbox(value=False, description='是否保存图像', disabled=False))

    def _probplot(self, column, start_date, end_date, boxcox, lmbda, save=False):
        start_date = pd.to_datetime(start_date)
        end_date = pd.to_datetime(end_date)
        probplot_df = self.loc[(self.index >= start_date) & (self.index <= end_date)][column]
        set_figsize(10, 5)
        if boxcox:
            stats.probplot(stats.boxcox(probplot_df, lmbda=lmbda), plot=plt)
        else:
            stats.probplot(probplot_df, plot=plt)
        if save:
            plt.savefig(f"参数{column}的PP图.png", dpi=600)

    def probplot(self):
        """
        单参数PP图

        :param str column: 分析参数
        :param str start_date: 开始日期
        :param str end_date: 结束日期
        :param bool boxcox: 是否进行box-cox变换
        :param float lmbda: box-cox变换的lmbda值
        :param bool save: 是否保存作图

        .. jupyter-execute::
            :hide-code:

            import ipywidgets as widgets
            from ipywidgets import interact_manual
            from datetime import datetime, timedelta
            interact_manual.opts['manual_name'] = '确认'

            columns = ['param_1', 'param_2', 'param_3', 'param_4']
            today = datetime.now()
            yesterday = datetime.now() - timedelta(1)

            def probplot_demo(column, start_date, end_date, boxcox, lmbda, save=False):
                print()

            interact_manual(probplot_demo,
                            column=widgets.Dropdown(options=columns, value=columns[0], description='参数: '),
                            start_date=widgets.DatePicker(value=yesterday, description='开始日期: '),
                            end_date=widgets.DatePicker(value=today, description='结束日期: '),
                            boxcox=widgets.Checkbox(value=False, description='是否进行box-cox变换', disabled=False),
                            lmbda=widgets.Text(value=None, description='lmbda值', disabled=False),
                            save=widgets.Checkbox(value=False, description='是否保存图像', disabled=False));
        """
        print('\n            单参数PP图')
        return interact_manual(self._probplot,
                               column=widgets.Dropdown(options=list(self.columns), value=self.columns[0],
                                                       description='参数: '),
                               start_date=widgets.DatePicker(value=self.index[0], description='开始日期: '),
                               end_date=widgets.DatePicker(value=self.index[-1], description='结束日期: '),
                               boxcox=widgets.Checkbox(value=False, description='是否进行box-cox变换', disabled=False),
                               lmbda=widgets.Text(value=None, description='lmbda值', disabled=False),
                               save=widgets.Checkbox(value=False, description='是否保存图像', disabled=False))

    def _scatterplot(self, column_1, column_2, start_date, end_date, save=False):
        start_date = pd.to_datetime(start_date)
        end_date = pd.to_datetime(end_date)
        scatterplot_df_1 = self.loc[(self.index >= start_date) & (self.index <= end_date)][column_1]
        scatterplot_df_2 = self.loc[(self.index >= start_date) & (self.index <= end_date)][column_2]
        set_figsize(10, 5)
        sns.scatterplot(x=scatterplot_df_1, y=scatterplot_df_2)
        if save:
            plt.savefig(f"参数{column_1}和{column_2}的散点图.png", dpi=600)

    def scatterplot(self):
        """
        双变量散点图

        :param str column_1: 分析参数1
        :param str column_2: 分析参数2
        :param str start_date: 开始日期
        :param str end_date: 结束日期
        :param bool save: 是否保存作图

        .. jupyter-execute::
            :hide-code:

            import ipywidgets as widgets
            from ipywidgets import interact_manual
            from datetime import datetime, timedelta
            interact_manual.opts['manual_name'] = '确认'

            columns = ['param_1', 'param_2', 'param_3', 'param_4']
            today = datetime.now()
            yesterday = datetime.now() - timedelta(1)

            def scatterplot_demo(column_1, column_2, start_date, end_date, save=False):
                print()

            interact_manual(scatterplot_demo,
                            column_1=widgets.Dropdown(options=columns, value=columns[0], description='X轴参数: '),
                            column_2=widgets.Dropdown(options=columns, value=columns[1], description='Y轴参数: '),
                            start_date=widgets.DatePicker(value=yesterday, description='开始日期: '),
                            end_date=widgets.DatePicker(value=today, description='结束日期: '),
                            save=widgets.Checkbox(value=False, description='是否保存图像', disabled=False));
        """
        print('\n            双参数散点图')
        return interact_manual(self._scatterplot,
                               column_1=widgets.Dropdown(options=list(self.columns), value=self.columns[0],
                                                         description='X轴参数: '),
                               column_2=widgets.Dropdown(options=list(self.columns), value=self.columns[1],
                                                         description='Y轴参数: '),
                               start_date=widgets.DatePicker(value=self.index[0], description='开始日期: '),
                               end_date=widgets.DatePicker(value=self.index[-1], description='结束日期: '),
                               save=widgets.Checkbox(value=False, description='是否保存图像', disabled=False))

    def _pairplot(self, pairplot_columns, start_date, end_date, save=False):
        start_date = pd.to_datetime(start_date)
        end_date = pd.to_datetime(end_date)
        pairplot_columns = list(pairplot_columns)
        pairplot_df = self.loc[(self.index >= start_date) & (self.index <= end_date)][pairplot_columns]
        set_figsize(10, 5)
        sns.pairplot(pairplot_df)
        if save:
            plt.savefig(f"参数{pairplot_columns}的两两关系图.png", dpi=600)

    def pairplot(self):
        """
        多参数的两两关系图

        :param list pairplot_columns: 分析参数列表
        :param str start_date: 开始日期
        :param str end_date: 结束日期
        :param bool save: 是否保存作图

        .. jupyter-execute::
            :hide-code:

            import ipywidgets as widgets
            from ipywidgets import interact_manual
            from datetime import datetime, timedelta
            interact_manual.opts['manual_name'] = '确认'

            columns = ['param_1', 'param_2', 'param_3', 'param_4']
            today = datetime.now()
            yesterday = datetime.now() - timedelta(1)

            def pairplot_demo(pairplot_columns, start_date, end_date, save=False):
                print()

            interact_manual(pairplot_demo,
                            pairplot_columns=widgets.SelectMultiple(options=columns, value=columns[0:2],
                                                                    description='多参数选择'),
                            start_date=widgets.DatePicker(value=yesterday, description='开始日期: '),
                            end_date=widgets.DatePicker(value=today, description='结束日期: '),
                            save=widgets.Checkbox(value=False, description='是否保存图像 ', disabled=False));
        """
        print('\n            多参数的两两关系图')
        return interact_manual(self._pairplot,
                               pairplot_columns=widgets.SelectMultiple(options=self.columns.tolist(),
                                                                       value=self.columns[0:2].tolist(),
                                                                       description='多参数选择'),
                               start_date=widgets.DatePicker(value=self.index[0], description='开始日期: '),
                               end_date=widgets.DatePicker(value=self.index[-1], description='结束日期: '),
                               save=widgets.Checkbox(value=False, description='是否保存图像', disabled=False))

    def _heatmap(self, column, num, start_date, end_date, save=False):
        set_figsize(10, 5)
        start_date = pd.to_datetime(start_date)
        end_date = pd.to_datetime(end_date)
        heatmap_df = self.loc[(self.index >= start_date) & (self.index <= end_date)]
        corr_largest = heatmap_df.corr().nlargest(num, column)[column]
        print(f"与参数{column}正相关性最强的{num}个参数:\n{corr_largest}")
        cols_largest = corr_largest.index
        sns.heatmap(heatmap_df[cols_largest].corr())
        if save:
            plt.savefig(f"与参数{column}正相关性最强的{num}个参数.png", dpi=600)
        corr_smallest = heatmap_df.corr().nsmallest(num, column)[column]
        print(f"与参数{column}负相关性最强的{num}个参数:\n{corr_smallest}")
        cols_smallest = corr_smallest.index
        sns.heatmap(heatmap_df[cols_smallest].corr())
        if save:
            plt.savefig(f"与参数{column}负相关性最强的{num}个参数.png", dpi=600)

    def heatmap(self):
        """
        相关性热点图

        :param str column: 分析参数
        :param int num: 展示与分析参数相关性最强的参数的数量
        :param str start_date: 开始日期
        :param str end_date: 结束日期
        :param bool save: 是否保存作图

        .. jupyter-execute::
            :hide-code:

            import ipywidgets as widgets
            from ipywidgets import interact_manual
            from datetime import datetime, timedelta
            interact_manual.opts['manual_name'] = '确认'

            columns = ['param_1', 'param_2', 'param_3', 'param_4']
            today = datetime.now()
            yesterday = datetime.now() - timedelta(1)

            def heatmap_demo(column, num, start_date, end_date, save=False):
                print()

            interact_manual(heatmap_demo,
                            column=widgets.Dropdown(options=columns, value=columns[0], description='参数: '),
                            num=widgets.IntSlider(value=10, min=1, max=50, step=1, description='数量', disabled=False),
                            start_date=widgets.DatePicker(value=yesterday, description='开始日期: '),
                            end_date=widgets.DatePicker(value=today, description='结束日期: '),
                            save=widgets.Checkbox(value=False, description='是否保存图像', disabled=False));
        """
        print('\n            相关性热点图')
        return interact_manual(self._heatmap,
                               column=widgets.Dropdown(options=list(self.columns), value=self.columns[0],
                                                       description='参数: '),
                               num=widgets.IntSlider(value=10, min=1, max=50, step=1,
                                                     description='数量', disabled=False),
                               start_date=widgets.DatePicker(value=self.index[0], description='开始日期: '),
                               end_date=widgets.DatePicker(value=self.index[-1], description='结束日期: '),
                               save=widgets.Checkbox(value=False, description='是否保存图像', disabled=False))



# -*- coding: utf-8 -*-
###############################################################################
# Copyright (c) 2014-2019 Mixlinker Networks Inc. <mixiot@mixlinker.com>
# All rights reserved.
#
# This program and the accompanying materials are made available under the
# terms of the Application License of Mixlinker Networks License and Mixlinker
# Distribution License which accompany this distribution.
#
# The Mixlinker License is available at
#    http://www.mixlinker.com/legal/license.html
# and the Mixlinker Distribution License is available at
#    http://www.mixlinker.com/legal/distribution.html
#
# Contributors:
#    Mixlinker Technical Team
###############################################################################
# Description : mavis核心功能
# Author : kanxiangyang

import pandas as pd
import ipywidgets as widgets
import boto3
import os
from ipywidgets import interact, interact_manual
from IPython.core.interactiveshell import InteractiveShell
from .transfer import read
from .plot import MavisPlot
from . import config

interact_manual.opts['manual_name'] = '确认'
InteractiveShell.ast_node_interactivity = 'all'


class Analysis(object):
    def __init__(self):
        self.whole_file = None
        self.fileframe = None
        self.directory = None
        self.count = None
        self.last_record = None
        self.max_length = None
        self.data = None
        self.preview_repository = None
        self.model_repository = None

    def select_preview(self, plot):
        mv = MavisPlot(self.data)
        return getattr(mv, self.preview_repository[plot])()

    def select_model(self, model):
        # 调用模型和数据
        module_mavis = __import__("mavis")
        model_class = getattr(module_mavis, config.model_repository[model])
        model_class(self.data)()

    def select_filename(self, preview):
        filename_split = self.directory.rsplit('/', 1)
        self.data = read(filename_split[0], filename_split[1])
        if preview:
            self.preview_repository = config.preview_repository
            return interact_manual(self.select_preview,
                                   plot=widgets.Dropdown(options=list(self.preview_repository.keys()),
                                                         value=list(self.preview_repository.keys())[0],
                                                         description='可视化库 '))
        else:
            self.model_repository = config.model_repository
            return interact_manual(self.select_model,
                                   model=widgets.Dropdown(options=list(self.model_repository.keys()),
                                                          value=list(self.model_repository.keys())[0],
                                                          description='算法库 '))

    def select_directory(self, directory):
        # 判断目录选择的递归函数回到了第几层
        if self.last_record:
            self.fileframe = self.whole_file
            for col in range(len(self.last_record)):
                if {directory, self.last_record[col]}.issubset(self.fileframe.iloc[:, col].unique()):
                    self.count = col
                    self.directory = os.path.join(*self.last_record[:col]) if col > 0 else None
                    for i in range(col):
                        self.fileframe = self.fileframe[self.fileframe[i] == self.last_record[i]]
                    break
                else:
                    pass
        if not self.directory:
            self.directory = directory
        else:
            self.directory = self.directory + '/' + directory
        self.fileframe = self.fileframe[self.fileframe[self.count] == directory]
        self.count += 1
        if not self.fileframe.iloc[:, self.count:].any(axis=None):
            self.last_record = self.directory.split('/')
            return interact_manual(self.select_filename,
                                   preview=widgets.Checkbox(value=False, description='是否预览数据', disabled=False))
        else:
            self.last_record = None
            directory_list = self.fileframe[self.count].unique().tolist()
            if '' in directory_list:
                directory_list.remove('')
            return interact(self.select_directory,
                            directory=widgets.Dropdown(options=directory_list, value=directory_list[0],
                                                       description='子目录 '))

    def __call__(self, ):
        self.fileframe = pd.DataFrame(
            [f['Key'].split('/') for f in boto3.client("s3").list_objects(Bucket='mixiot')['Contents']])
        self.whole_file = self.fileframe
        self.max_length = self.whole_file.shape[1]
        self.count = 0
        directory_list = self.fileframe[self.count].unique().tolist()
        return interact(self.select_directory,
                        directory=widgets.Dropdown(options=directory_list, value=directory_list[0],
                                                   description='数据仓库目录 '))



# -*- coding: utf-8 -*-
###############################################################################
# Copyright (c) 2014-2019 Mixlinker Networks Inc. <mixiot@mixlinker.com>
# All rights reserved.
#
# This program and the accompanying materials are made available under the
# terms of the Application License of Mixlinker Networks License and Mixlinker
# Distribution License which accompany this distribution.
#
# The Mixlinker License is available at
#    http://www.mixlinker.com/legal/license.html
# and the Mixlinker Distribution License is available at
#    http://www.mixlinker.com/legal/distribution.html
#
# Contributors:
#    Mixlinker Technical Team
###############################################################################
# Description : 基于深度学习的锅炉生产参数预测模型训练
# Author : zhaoguangjun

import numpy as np
import pandas as pd
import os
import ipywidgets as widgets
from ipywidgets import interact_manual
from datetime import datetime, timedelta
from IPython.core.interactiveshell import InteractiveShell
from itertools import product, chain
from pandas.plotting import register_matplotlib_converters
import torch
import torch.nn as nn
from torch.utils import data
from .net import Net
from .data_read import Read_data
import torch.optim as optim
from .config import Config
from .draw import Draw

register_matplotlib_converters(np.datetime64)

pd.options.display.max_rows = 30
pd.options.display.max_columns = 25
interact_manual.opts['manual_name'] = '确认'
InteractiveShell.ast_node_interactivity = 'all'


def weights_init(m):
    classname = m.__class__.__name__
    if classname.find('Conv') != -1:
        m.weight.data.normal_(0.0, 0.02)
    elif classname.find('BatchNorm') != -1:
        m.weight.data.normal_(1.0, 0.02)
        m.bias.data.fill_(0)


def adjust_learning_rate(optimizer, num_epoch, factor=0.9, step=10):
    """Sets the learning rate to the initial LR decayed by "factor" every "step" epochs"""
    for param_group in optimizer.param_groups:
        param_group['lr'] = param_group['lr'] * (factor ** (num_epoch // step))


class Boiler(object):
    def __init__(self, data):
        self.data = data

    def __call__(self):
        print("数据预处理中")
        self.preprocessing()
        print("数据加工完成")
        return interact_manual(self.set_tuning,
                               tunning=widgets.Checkbox(value=False, description='启用超参数调试', disabled=False))

    def preprocessing(self):
        pass

    def moment_set(self, batch_size, epochs):
        return interact_manual(self.tuning,
                               batch_size_range=batch_size,
                               epochs=epochs,
                               learning_rate_range=widgets.FloatRangeSlider(
                                   value=[0.01, 0.1], min=1e-8, max=1, step=0.01, description='学习率',
                                   disabled=False),
                               moment_range=widgets.FloatRangeSlider(value=[0.5, 0.9922], min=0.5, max=0.9922,
                                                                     step=0.01,
                                                                     description='动量', disabled=False),
                               hidden_dim_range=widgets.IntRangeSlider(value=[128, 4096], min=128, max=4096, step=512,
                                                                       description='模型隐藏层', disabled=False),
                               )

    def set_tuning(self, tunning):
        if tunning:
            return interact_manual(self.tuning,
                                   batch_size_range=widgets.SelectionRangeSlider(
                                       options=np.linspace(2, 128, 64, dtype=np.int64),
                                       index=(0, 63),
                                       description='单次数据读取量',
                                       disabled=False),
                                   epochs=widgets.IntText(value=30, description='遍历数据集次数', disabled=False),
                                   learning_rate_range=widgets.FloatRangeSlider(
                                       value=[1e-8, 0.99], min=1e-8, max=1, step=0.00001,
                                       description='学习率',
                                       disabled=False),
                                   moment_range=widgets.FloatRangeSlider(value=[0.5, 0.9922], min=0.5, max=0.9922,
                                                                         step=0.01,
                                                                         description='动量', disabled=False),
                                   hidden_dim_range=widgets.IntRangeSlider(value=[128, 4096], min=128, max=4096,
                                                                           step=512,
                                                                           description='模型隐藏层',
                                                                           disabled=False),
                                   inkeys=widgets.SelectMultiple(
                                       options=self.data.columns.tolist(),
                                       value=self.data.columns[2:3].tolist(),
                                       description='输入数据',
                                       disabled=False
                                   ),
                                   outkeys=widgets.SelectMultiple(
                                       options=self.data.columns.tolist(),
                                       value=self.data.columns[3:4].tolist(),
                                       description='输出数据',
                                       disabled=False
                                   ),
                                   )
        else:
            return interact_manual(self.fit,
                                   batch_size=widgets.IntText(value=10, description='单次数据读取',
                                                              disabled=False),
                                   epochs=widgets.IntText(value=30, description='遍历数据集次数', disabled=False),
                                   learning_rate=widgets.FloatText(value=0.1, description='学习率',
                                                                   disabled=False),
                                   moment=widgets.FloatText(value=0.9, description='动量', disabled=False),
                                   hidden_dim=widgets.IntText(value=1024, description='模型隐藏层',
                                                              disabled=False),
                                   inkeys=widgets.SelectMultiple(
                                       options=self.data.columns.tolist(),
                                       value=self.data.columns[2:3].tolist(),
                                       description='输入数据',
                                       disabled=False
                                   ),
                                   outkeys=widgets.SelectMultiple(
                                       options=self.data.columns.tolist(),
                                       value=self.data.columns[3:4].tolist(),
                                       description='输出数据',
                                       disabled=False
                                   ),
                                   )

    def tuning(self, batch_size_range, epochs, learning_rate_range, moment_range, hidden_dim_range, inkeys, outkeys):
        start_time = datetime.now()
        batch_size_list = chain(range(batch_size_range[0], batch_size_range[1], 2), (batch_size_range[1],))
        learning_rate_list = chain(np.arange(learning_rate_range[0], learning_rate_range[1], 0.01),
                                   (learning_rate_range[1],))
        hidden_dim_list = chain(range(hidden_dim_range[0], hidden_dim_range[1], 1), (hidden_dim_range[1],))
        moment_list = chain(np.arange(moment_range[0], moment_range[1], 0.01), (moment_range[1],))

        # cores = multiprocessing.cpu_count()
        # p = multiprocessing.Pool(cores)
        # product_list = product([epochs], learning_rate_list, batch_size_list, moment_list, hidden_dim_list)
        # print(product_list)
        # results = p.starmap(self.train, product_list)
        # p.close()
        # p.join()

        results = []
        for lr in learning_rate_list:
            for batch in batch_size_list:
                for moment in moment_list:
                    for hidd in hidden_dim_list:
                        results.append(self.train(epochs, lr, batch, moment, hidd, inkeys, outkeys))
        results_df = pd.DataFrame(results, columns=['训练集结果', '验证集结果', '遍历数据集次数', '学习率',
                                                    '单次数据读取量', '动量'])
        end_time = datetime.now()
        print('总耗时: ', end_time - start_time)
        results_df.to_csv('results_df.csv')
        return results_df

    def train(self, epochs, learning_rate, batch_size, moment, hidden_dim, inkeys, outkeys, l2=False):
        """
        此处的输入参数需要和tuning中的product_list一一对应.
        且此处optimizer中weight_decay已经被固定,如有需要可以传参.
        损失函数loss_fn被固定,可改写传参.
        :param epochs:
        :param learning_rate:
        :param batch_size:
        :param moment:
        :return:
        """

        print(inkeys, len(inkeys))
        print(outkeys, len(outkeys))
        net = Net(in_class=len(inkeys), hidden_dim=hidden_dim, out_class=len(outkeys))
        train_data = Read_data(self.data, mode="train", in_keys=inkeys, out_keys=outkeys)
        train_loader = data.DataLoader(train_data, batch_size=batch_size, shuffle=False, num_workers=4)
        optimizer = optim.SGD(params=net.parameters(), lr=learning_rate, momentum=moment, weight_decay=2e-5)
        lr_schedule = optim.lr_scheduler.ReduceLROnPlateau(optimizer, mode='min', factor=0.3, verbose=True, patience=5)
        loss_fn = nn.MSELoss()
        lowest_loss = 50000
        net.train()
        for epoch in range(epochs):
            total_loss = 0
            for t, (x, y) in enumerate(train_loader):
                optimizer.zero_grad()
                x = torch.unsqueeze(x, 2).float()
                output = net(x)
                # L2正则化
                if l2:
                    regularization_loss = 0
                    for param in net.parameters():
                        regularization_loss += torch.sum(abs(param))
                    loss = loss_fn(output, y.float()) + 0.5 * regularization_loss
                else:
                    loss = loss_fn(output, y.float())
                loss.backward()
                optimizer.step()
                total_loss += loss
                avg_loss = total_loss / (t + 1)
                if (t + 1) % 10 == 0:
                    print('epoch : %d / %d, iter: %d, average loss is: %.6f, batch loss: %.6f' % (
                        epoch + 1, epochs, t, avg_loss, loss))
            adjust_learning_rate(optimizer, epoch + 1, step=10)
            if avg_loss < lowest_loss:
                lowest_loss = avg_loss
                print('saving model .......')
                if not os.path.exists(Config.model_path):
                    os.makedirs(Config.model_path)
                torch.save(net, os.path.join(Config.model_path, "best_model.pkl"))
                print('finished saving model!')
            val_loss = self.validation(net, batch_size, inkeys, outkeys)
            lr_schedule.step(val_loss)
        return [avg_loss, val_loss] + [epochs, learning_rate, batch_size, moment, hidden_dim]

    def validation(self, net, batch_size, inkeys, outkeys):
        val_data = Read_data(self.data, mode='val', inkeys=inkeys, outkeys=outkeys)
        val_loader = data.DataLoader(val_data, batch_size=batch_size, shuffle=False, num_workers=4)
        net.eval()
        total_loss = 0
        for i, (x, y) in enumerate(val_loader):
            with torch.no_grad():
                x = torch.unsqueeze(x, 2).float()
                out = net(x)
                loss = self.loss_func(out, y.float())
                total_loss += loss.item()
        val_loss = total_loss / (len(val_data))
        print("validation loss is : %.5f", val_loss)
        return val_loss

    def fit(self, batch_size, epochs, learning_rate, moment, hidden_dim, inkeys, outkeys):
        return self.train(epochs=epochs, learning_rate=learning_rate, batch_size=batch_size, moment=moment,
                          hidden_dim=hidden_dim, inkeys=inkeys, outkeys=outkeys)


# -*- coding: utf-8 -*-
###############################################################################
# Copyright (c) 2014-2019 Mixlinker Networks Inc. <mixiot@mixlinker.com>
# All rights reserved.
#
# This program and the accompanying materials are made available under the
# terms of the Application License of Mixlinker Networks License and Mixlinker
# Distribution License which accompany this distribution.
#
# The Mixlinker License is available at
#    http://www.mixlinker.com/legal/license.html
# and the Mixlinker Distribution License is available at
#    http://www.mixlinker.com/legal/distribution.html
#
# Contributors:
#    Mixlinker Technical Team
###############################################################################
# Description : 自定义动态画图模块
# Author : zhaoguangjun

import matplotlib.pyplot as plt
from matplotlib.animation import FuncAnimation
class Draw():
    def __init__(self, x, y):
        """

        :param data:
        """
        self.x = x
        self.y = y
        self.fig, self.ax = plt.subplots()
        self.xdata, self.ydata = [], []
        self.ln = self.ax.plot([], [], 'r-', animated=False)

        super(Draw, self).__init__()

    # def init_func(self):
    #     self.ax.set_xlim(0, 2 * np.pi)
    #     self.ax.set_ylim(-1, 1)
    #     return self.ln

    def __call__(self):
        return self.plot_((self.x, self.y))

    def update(self, data):
        print("data is:", data[0], data[1])
        self.xdata.append(data[0])
        self.ydata.append(data[1])
        self.ln.set_data(self.xdata, self.ydata)
        return self.ln

    def plot_(self, n):
        # 这里的frames在调用update函数时会将frames作为实参传递给“data”
        ani = FuncAnimation(fig=self.fig, func=self.update, frames=n)
        plt.show()



# -*- coding: UTF-8 -*-

import ipywidgets as widgets
from ipywidgets import interact_manual
from IPython.core.interactiveshell import InteractiveShell
import numpy as np
import pandas as pd
import matplotlib as mpl
import matplotlib.pyplot as plt
import seaborn as sns
from itertools import product
import multiprocessing
from sklearn.model_selection import train_test_split

# matplotlib中文化支持
mpl.rcParams['font.sans-serif'] = ['simhei']
mpl.rcParams['font.family'] = 'sans-serif'
mpl.rcParams['axes.unicode_minus'] = False

interact_manual.opts['manual_name'] = '确认'
InteractiveShell.ast_node_interactivity = 'all'

# 超参数调试
hyper_parameters = {
    'para1': {'description': '参数1', 'default': 0.1, 'range': np.float_power(10, range(-2, 3))},
    'para2': {'description': '参数2', 'default': 256, 'range': np.int32(np.exp2(range(7, 11)))},
    'para3': {'description': '参数3', 'default': 500, 'range': range(100, 600, 100)},
    'para4': {'description': '参数4', 'default': 0.999, 'range': [0.9, 0.99, 0.999, 0.9999]}
}

# fit交互模块
fit_kwargs = {k: widgets.Text(value=str(v['default']), description=v['description'], disabled=False) for k, v in
              hyper_parameters.items()}

# tuing交互模块
tuning_kwargs = {
    k + '_list': widgets.SelectionRangeSlider(options=[(v['range'][i], i) for i in range(len(v['range']))],
                                              index=(0, 1), description=v['description'],
                                              disabled=False) for k, v in hyper_parameters.items()}


class ModelDemo(object):
    def __init__(self, data):
        self.data = data
        self.mean = None
        self.std = None
        self.interval = 60

    def __call__(self):
        return interact_manual(self.set_tuing,
                               tuning=widgets.Checkbox(value=False, description='启用超参数调试', disabled=False)
                               )

    # 数据预处理
    def preprocessing(self):
        self.data = self.data.dropna(axis=1, how='all')
        self.data = self.data.dropna(axis=0, how='any')
        self.mean = self.data.mean(axis=0)
        std = self.data.std(axis=0)
        constant_col = std[std < 1e-6].index.tolist()
        std[constant_col] = 1
        self.std = std
        self.data -= self.mean
        self.data /= self.std

    def set_tuing(self, tuning):
        if tuning:
            return interact_manual(self.tuning, **tuning_kwargs)
        else:
            return interact_manual(self.fit, **fit_kwargs)

    def train(self, para1, para2, para3, para4):
        x_train, x_validation = train_test_split(self.data, test_size=0.2, shuffle=False)
        compare_vector_train = np.random.rand(1, 4) + np.log10(x_train.shape[0])
        compare_vector_validation = np.random.rand(1, 4) + np.log10(x_validation.shape[0])
        parameter_vector = [np.log10(para1), para2 // 128, para3 // 100, np.log10(1 - para4)]
        result_train = np.linalg.norm(compare_vector_train - parameter_vector)
        result_validation = np.linalg.norm(compare_vector_validation - parameter_vector)
        return [result_train, result_validation] + [para1, para2, para3, para4]

    def tuning(self, para1_list, para2_list, para3_list, para4_list):
        for k in hyper_parameters.keys():
            v = locals()[k + '_list']
            hyper_parameters[k].update({'list': hyper_parameters[k]['range'][v[0]:v[1] + 1]})
            hyper_parameters[k].update({'list_length': v[1] - v[0] + 1})
        hyper_df = pd.DataFrame.from_dict(hyper_parameters, orient='index')
        if hyper_df[hyper_df['list_length'] != 1].shape[0] > 2:
            print('一口吃不成胖子，一步跨不到天边，最多同时调两个参数，谢谢')
            raise Exception
        cores = multiprocessing.cpu_count()
        p = multiprocessing.Pool(cores)
        product_list = product(*hyper_df['list'].tolist())
        results = p.starmap(self.train, product_list)
        p.close()
        p.join()
        tuning_params = hyper_df[hyper_df['list_length'] != 1]['description'].tolist()
        columns = ['训练集结果', '验证集结果'] + hyper_df['description'].tolist()
        results_df = pd.DataFrame(results, columns=columns)
        if len(tuning_params) == 2:
            nums_1 = hyper_df.loc[hyper_df[hyper_df['description'] == tuning_params[0]].index[0], 'list_length']
            mpl.rcParams["figure.figsize"] = [10, 6 * nums_1]
            fig_1, axes_1 = plt.subplots(nrows=nums_1)
            for idx, (value, split_df) in enumerate(results_df.groupby(tuning_params[0])):
                split_df.index = split_df[tuning_params[1]]
                split_df = split_df[['训练集结果', '验证集结果']]
                ax = sns.lineplot(data=split_df, markers=True, ax=axes_1[idx])
                ax.grid(True)
                ax.set_title(f"{tuning_params[0]}: {value}")
            nums_2 = hyper_df.loc[hyper_df[hyper_df['description'] == tuning_params[1]].index[0], 'list_length']
            mpl.rcParams["figure.figsize"] = [10, 6 * nums_2]
            fig_2, axes_2 = plt.subplots(nrows=nums_2)
            for idx, (value, split_df) in enumerate(results_df.groupby(tuning_params[1])):
                split_df.index = split_df[tuning_params[0]]
                split_df = split_df[['训练集结果', '验证集结果']]
                ax = sns.lineplot(data=split_df, markers=True, ax=axes_2[idx])
                ax.grid(True)
                ax.set_title(f"{tuning_params[1]}: {value}")
        else:
            mpl.rcParams["figure.figsize"] = [12, 8]
            results_df.index = results_df[tuning_params[0]]
            results_df = results_df[['训练集结果', '验证集结果']]
            ax = sns.lineplot(data=results_df, markers=True)
            ax.grid(True)
        return results_df

    def fit(self, para1, para2, para3, para4):
        print('DEMO 演示')
        print(self.data.head(), para1, para2, para3, para4)



# -*- coding: utf-8 -*-
###############################################################################
# Copyright (c) 2014-2019 Mixlinker Networks Inc. <mixiot@mixlinker.com>
# All rights reserved.
#
# This program and the accompanying materials are made available under the
# terms of the Application License of Mixlinker Networks License and Mixlinker
# Distribution License which accompany this distribution.
#
# The Mixlinker License is available at
#    http://www.mixlinker.com/legal/license.html
# and the Mixlinker Distribution License is available at
#    http://www.mixlinker.com/legal/distribution.html
#
# Contributors:
#    Mixlinker Technical Team
###############################################################################
# Description : 基于Iforest的异常检测算法
# Author : kanxiangyang

import numpy as np
import pandas as pd
import ipywidgets as widgets
from ipywidgets import interact_manual
import matplotlib as mpl
import matplotlib.pyplot as plt
import seaborn as sns
import multiprocessing
from datetime import datetime, timedelta
from IPython.core.interactiveshell import InteractiveShell
from sklearn.model_selection import train_test_split
from sklearn.ensemble import IsolationForest
from itertools import product, chain
from ast import literal_eval
import _pickle as cPickle
from pandas.plotting import register_matplotlib_converters
# import mavis
# import threading
# import requests
# import json
# import time
# from IPython.display import clear_output


register_matplotlib_converters(np.datetime64)

pd.options.display.max_rows = 30
pd.options.display.max_columns = 25
interact_manual.opts['manual_name'] = '确认'
InteractiveShell.ast_node_interactivity = 'all'


class IForest(object):
    def __init__(self, data):
        self.data = data
        self.mean = None
        self.std = None
        self.interval = 60
        self.result = pd.DataFrame()

    def __call__(self):
        print("数据预处理中")
        self.preprocessing()
        print("数据加工完成")
        return interact_manual(self.set_tuing,
                               tuning=widgets.Checkbox(value=False, description='启用超参数调试', disabled=False))

    def preprocessing(self):
        self.data = self.data.dropna(axis=1, how='all')
        self.data = self.data.dropna(axis=0, how='any')
        self.mean = self.data.mean(axis=0)
        std = self.data.std(axis=0)
        constant_col = std[std < 1e-6].index.tolist()
        std[constant_col] = 1
        self.std = std
        self.data -= self.mean
        self.data /= self.std

    def set_tuing(self, tuning):
        if tuning:
            return interact_manual(self.tuning,
                                   n_estimators_range=widgets.IntRangeSlider(
                                       value=[100, 100], min=100, max=1000, step=100, description='基学习器数量: ',
                                       disabled=False),
                                   max_samples_range=widgets.SelectionRangeSlider(
                                       options=list(np.exp2(range(4, 12)).astype('int')), index=(4, 6),
                                       description='最大叶样本数: ', disabled=False),
                                   max_features_range=widgets.FloatRangeSlider(
                                       value=[0.5, 0.9], min=0, max=1, step=0.1, description='最大叶特征比例: ',
                                       disabled=False),
                                   contamination=widgets.FloatLogSlider(
                                       value=1e-3, min=-6, max=-1, base=10, description='异常比例: ', disabled=False)
                                   )
        else:
            return interact_manual(self.fit,
                                   n_estimators=widgets.Text(value='100', description='基学习器数量', disabled=False),
                                   max_samples=widgets.Text(value='auto', description='最大叶样本数', disabled=False),
                                   max_features=widgets.Text(value='1.0', description='最大叶特征比例', disabled=False),
                                   contamination=widgets.Text(value='0.1', description='异常比例', disabled=False)
                                   )

    def train(self, n_estimators, max_samples, max_features, contamination):
        x_train, x_validation = train_test_split(self.data, test_size=0.2, shuffle=False)
        model = IsolationForest(n_estimators=n_estimators, max_samples=n_estimators, max_features=max_features,
                                contamination=contamination, behaviour='new',
                                random_state=30, verbose=True)
        model.fit(x_train)
        score_anomalies_train = model.decision_function(x_train)
        score_anomalies_validation = model.decision_function(x_validation)
        score_train = pd.Series(score_anomalies_train, index=x_train.index)
        score_validation = pd.Series(score_anomalies_validation, index=x_validation.index)
        ewm_score_train = (x_train - x_train.ewm(com=9).mean()).apply(lambda x: np.linalg.norm(x), axis=1)
        ewm_score_validation = (x_validation - x_validation.ewm(com=9).mean()).apply(lambda x: np.linalg.norm(x),
                                                                                     axis=1)
        metrics_train = np.corrcoef(ewm_score_train, score_train)[0, 1]
        metrics_validation = np.corrcoef(ewm_score_validation, score_validation)[0, 1]
        return [metrics_train, metrics_validation] + [n_estimators, max_samples, max_features, contamination]

    def tuning(self, n_estimators_range, max_samples_range, max_features_range, contamination):
        start_time = datetime.now()
        n_estimators_list = chain(range(n_estimators_range[0], n_estimators_range[1], 100), (n_estimators_range[1],))
        max_features_list = chain(np.arange(max_features_range[0], max_features_range[1], 0.1),
                                  (max_features_range[1],))

        cores = multiprocessing.cpu_count()
        p = multiprocessing.Pool(cores)
        product_list = product(n_estimators_list, max_samples_range, max_features_list, [contamination])
        results = p.starmap(self.train, product_list)
        p.close()
        p.join()
        results_df = pd.DataFrame(results, columns=['训练集结果', '验证集结果', '基学习器数量', '最大叶样本数',
                                                    '最大叶特征比例', '异常比例'])
        mpl.rcParams["figure.figsize"] = [8, 8]
        g = sns.FacetGrid(results_df, col='最大叶样本数', col_wrap=1)
        g.map(plt.plot, '最大叶特征比例', '训练集结果', label='训练集', marker=".", color='b', alpha=.7).set_axis_labels(
            '最大叶特征比例', '结果')
        g.map(plt.plot, '最大叶特征比例', '验证集结果', label='验证集', marker=".", color='r', alpha=.7).set_axis_labels(
            '最大叶特征比例', '结果')
        g.add_legend()
        end_time = datetime.now()
        print('总耗时: ', end_time - start_time)
        results_df.to_csv('results_df.csv')
        return results_df

    def fit(self, n_estimators, max_samples, max_features, contamination):
        n_estimators = literal_eval(n_estimators)
        if max_samples is not 'auto':
            max_samples = literal_eval(max_samples)
        max_features = literal_eval(max_features)
        contamination = literal_eval(contamination)
        model = IsolationForest(n_estimators=n_estimators, max_samples=max_samples, max_features=max_features,
                                contamination=contamination, n_jobs=-1, behaviour='new', random_state=42, verbose=True)
        model.fit(self.data)
        with open('iforest.pkl', 'wb') as f:
            cPickle.dump(model, f)

    # def predict(self):
    #
    #
    # def _predict(self, ):
    #     while True:
    #         clear_output(wait=True)
    #         plt.figure(figsize=(16, 8))
    #         mavis.get_data(url="")
    #         plt.plot(self.result)
    #         plt.title('实时异常分数')
    #         plt.grid(True)
    #         plt.show()
    #         time.sleep(self.interval)

    # def pump(self, url, authorization, device, interval):
    #     self.interval = literal_eval(interval)
    #     time_now = datetime.now()
    #     start_time = (time_now - timedelta(seconds=self.interval)).strftime('%Y-%m-%d %H:%M:%S')
    #     end_time = time_now.strftime('%Y-%m-%d %H:%M:%S')
    #     parameter = {'equipment_id': device, 'page_size': '10', 'page_index': '1',
    #                  'end_time': str(end_time), 'start_time': str(start_time), 'keys': self.keys}
    #     response = requests.post(url=url, headers={'Authorization': authorization}, data=parameter)
    #     json_data = json.loads(response.content)['result']['data']
    #     if not json_data:
    #         print('查询为空')
    #     response.close()
    #     time_index = pd.to_datetime([j['time'] for j in json_data])
    #     df = pd.DataFrame([j['value'] for j in json_data], index=time_index, columns=self.columns)
    #     df -= self.mean
    #     df /= self.std
    #     df = df.mean(axis=0).to_frame().T
    #     df.index = [time_index[0]]
    #     with open('iforest.pkl', 'rb') as f:
    #         model = cPickle.load(f)
    #     self.result = self.result.append(pd.Series(model.decision_function(df), index=[time_index[0]]))
    #     self.result.name = 'score'
    #     print('success:', time.ctime())
    #
    # def timer(self, url, authorization, device, interval):
    #     global t
    #     t = threading.Timer(interval=self.interval, function=self.pump,
    #                         kwargs={url: url, authorization: authorization, device: device, interval: interval})
    #     t.start()
    #
    # def interact_pump(self):
    #     return interact_manual(self.timer,
    #                            url=widgets.Text(
    #                                value='http://pro.sunevap.mixiot.top/pro/equipment/public_get_mosaic_by_key',
    #                                description='请求地址', disabled=False),
    #                            authorization=widgets.Text(value='bear xxxxx', description='认证', disabled=False),
    #                            device=widgets.Text(value='', description='设备ID', disabled=False),
    #                            interval=widgets.Text(value='10', description='时间间隔/s', disabled=False)
    #                            )
    #
    # def live_plot(self):
    #     clear_output(wait=True)
    #     plt.figure(figsize=(16, 8))
    #     plt.plot(self.result)
    #     plt.title('实时异常分数')
    #     plt.grid(True)
    #     plt.show()
    #
    # def predict(self):
    #     while True:
    #         self.live_plot()
    #         time.sleep(self.interval)



# 数据预览库
preview_repository = {
    '数据统计量': 'summary',
    '参数缺失值比例图': 'naplot',
    '单参数趋势图': 'timeplot',
    '单参数分布图': 'distplot',
    '单参数PP图': 'probplot',
    '双参数散点图': 'scatterplot',
    '多参数关系图': 'pairplot',
    '多参数相关性热点图': 'heatmap'
}

# 算法库
model_repository = {
    'DEMO演示算法': 'ModelDemo',
    '锅炉生产参数预测的神经网络算法': 'Boiler',
    '锅炉异常检测算法': 'IForest'

}

# 平台库
platform_repository = {
    '康普斯': 'kps',
    '瑞升华': 'sunevap',
    '精旺': 'jingwang',
    '先创': 'xianchuang',
    '方块': 'fangkuai',
    '济柴': 'emiot',
    '演示': 'demo'
}


# Mixiot接口
class API(object):
    def __init__(self, platform):
        self.platform = platform

    def apix_url(self):
        if self.platform == 'emiot':
            return "http://emiot.cnpc.com.cn:8008/pro/equipment/public_get_mosaic_by_key"
        else:
            return f"http://pro.{self.platform}.mixiot.top/v1/apix/mosaicByKey"

    def apiq_login_url(self):
        if self.platform == 'emiot':
            return "http://emiot.cnpc.com.cn:8009/api/login"
        else:
            return f"http://admin.{self.platform}.mixiot.top/api/login"

    def apiq_info_url(self):
        if self.platform == 'emiot':
            return "http://emiot.cnpc.com.cn:8009/api/equipment/menu_list"
        else:
            return f"http://admin.{self.platform}.mixiot.top/api/equipment/get_menu_list"

    def apiq_pro_url(self):
        if self.platform == 'emiot':
            return f"http://emiot.cnpc.com.cn:8009/api/pro/getMapping"
        else:
            return f"http://admin.{self.platform}.mixiot.top/api/pro/getMapping"

    def influx_export_url(self):
        if self.platform == 'emiot':
            return f"http://emiot.cnpc.com.cn:9009/export/export_data"
        else:
            return f"http://{self.platform}.mixiot.top:9009/export/export_data"


# 数据重采样方法
resample_method = \
    {
        '降采样取最新一条数据': 'first',
        '降采样取最早一条数据': 'last',
        '降采样取均值': 'mean',
        '降采样取求和': 'sum',
        '升采样取最近一条数据': 'nearest',
        '升采样取前一条数据': 'ffill',
        '升采样取后一条数据': 'bfill'
    }

# Mixiot S3数据桶
bucket_name = 'mixiot'
aws_access_key_id = "AKIATL43NOIHFVTCHE5T"
aws_secret_access_key = "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"


# -*- coding: utf-8 -*-
###############################################################################
# Copyright (c) 2014-2019 Mixlinker Networks Inc. <mixiot@mixlinker.com>
# All rights reserved.
#
# This program and the accompanying materials are made available under the
# terms of the Application License of Mixlinker Networks License and Mixlinker
# Distribution License which accompany this distribution.
#
# The Mixlinker License is available at
#    http://www.mixlinker.com/legal/license.html
# and the Mixlinker Distribution License is available at
#    http://www.mixlinker.com/legal/distribution.html
#
# Contributors:
#    Mixlinker Technical Team
###############################################################################
# Description : 锅炉生产参数预测模型框架
# Author : zhaoguangjun

import torch.nn as nn
from .config import Config

class Net(nn.Module):
    def __init__(self, in_class=None, hidden_dim=None, out_class=None):
        self.in_class = in_class
        self.out_class = out_class
        self.hidden_dim = hidden_dim
        super(Net, self).__init__()

        self.conv1 = nn.Conv1d(in_channels=self.in_class, out_channels=self.hidden_dim, kernel_size=1, stride=1)
        self.bn1 = nn.BatchNorm1d(self.hidden_dim)
        self.relu1 = nn.LeakyReLU(inplace=True)

        self.conv2 = nn.Conv1d(in_channels=self.hidden_dim, out_channels=self.hidden_dim, kernel_size=1, stride=1)
        self.bn2 = nn.BatchNorm1d(self.hidden_dim)
        self.relu2 = nn.LeakyReLU(inplace=True)

        self.conv3 = nn.Conv1d(in_channels=self.hidden_dim, out_channels=self.out_class, kernel_size=1, stride=1)

    def forward(self, x):
        out = self.conv1(x)
        out = self.bn1(out)
        out = self.relu1(out)

        out = self.conv2(out)
        out = self.bn2(out)
        out = self.relu2(out)

        output = self.conv3(out).view(out.size(0), -1)
        return output



# -*- coding: utf-8 -*-
###############################################################################
# Copyright (c) 2014-2019 Mixlinker Networks Inc. <mixiot@mixlinker.com>
# All rights reserved.
#
# This program and the accompanying materials are made available under the
# terms of the Application License of Mixlinker Networks License and Mixlinker
# Distribution License which accompany this distribution.
#
# The Mixlinker License is available at
#    http://www.mixlinker.com/legal/license.html
# and the Mixlinker Distribution License is available at
#    http://www.mixlinker.com/legal/distribution.html
#
# Contributors:
#    Mixlinker Technical Team
###############################################################################
# Description : 自定义数据获取模块
# Author : zhaoguangjun

import torch
from torch.utils import data
import numpy as np
from sklearn.model_selection import train_test_split


class Read_data(data.Dataset):
    def __init__(self, data, max_read_len=None, mode="train", in_keys=None, out_keys=None):
        self.max_read = max_read_len
        self.in_keys = in_keys
        self.out_keys = out_keys
        self.mode = mode
        if self.max_read:
            self.in_data = data.loc[:self.max_read, self.in_keys]
            self.out_data = data.loc[:self.max_read, self.out_keys]
        else:
            self.in_data = data.loc[:, self.in_keys]
            self.out_data = data.loc[:, self.out_keys]

        self.train_data, self.val_data, self.train_label, self.val_label = train_test_split(self.in_data, self.out_data,
                                                                                            test_size=0.33,
                                                                                            random_state=2)
        if self.mode == "train":
            self.in_data = self.train_data
            self.out_data = self.train_label
        else:
            self.in_data = self.val_data
            self.out_data = self.val_label
        self.in_data = self._pd2tensor(self.in_data, self.in_keys)
        self.out_data = self._pd2tensor(self.out_data, self.out_keys)

    def _pd2tensor(self, np_data, keys):
        tensor_map = torch.tensor(np.array(np_data[keys[0]]).reshape(-1, 1))
        for key in keys[1:]:
            tensor_map = torch.cat((tensor_map, torch.tensor(np.array(np_data[key]).reshape(-1, 1))), 1)
        return tensor_map

    def __getitem__(self, index):
        return self.in_data[index], self.out_data[index]

    def __len__(self):
        return self.in_data.size(0)

# Copyright (c) 2014-2019 Mixlinker Networks Inc. <mixiot@mixlinker.com>
# All rights reserved.
#
# This program and the accompanying materials are made available under the
# terms of the Application License of Mixlinker Networks License and Mixlinker
# Distribution License which accompany this distribution.
#
# The Mixlinker License is available at
#    http://www.mixlinker.com/legal/license.html
# and the Mixlinker Distribution License is available at
#    http://www.mixlinker.com/legal/distribution.html
#
# Contributors:
#    Mixlinker Technical Team
###############################################################################
# Date : 2019-9-5
# Author : Zhao Guangjun


# import torch
# from torch.utils import data
# import numpy as np
# from config import Config
# import pandas as pd
#
#
# def pd2tensor(data, keys):
#     tensor_map = torch.tensor(np.array(data[keys[0]]).reshape(-1, 1))
#     for key in keys[1:]:
#         tensor_map = torch.cat((tensor_map, torch.tensor(np.array(data[key]).reshape(-1, 1))), 1)
#     return tensor_map
#
#
# class Read_data(data.Dataset):
#     def __init__(self, file_name=None, mode='train', max_read_len=None, in_keys=Config.in_keys,
#                  out_keys=Config.out_keys):
#         """
#
#         :param file_name:
#         :param mode:
#         :param map_name:
#         :param max_read_len:
#         :param in_keys:
#         :param out_keys:
#         """
#         self.file = file_name
#         self.mode = mode
#         self.max_read = max_read_len
#         self.in_keys = in_keys
#         self.out_keys = out_keys
#         super(Read_data, self).__init__()
#
#         if self.max_read:
#             self.in_data = pd.read_csv(self.file, sep=',', nrows=self.max_read, usecols=self.in_keys)
#             self.in_data = pd2tensor(self.in_data, self.in_keys)
#             if self.mode != "test":
#                 self.out_data = pd.read_csv(self.file, sep=',', nrows=self.max_read, usecols=self.out_keys)
#                 self.out_data = pd2tensor(self.out_data, self.out_keys)
#         else:
#             self.in_data = pd.read_csv(self.file, sep=',', usecols=self.in_keys)
#             self.in_data = pd2tensor(self.in_data, self.in_keys)
#             if self.mode != "test":
#                 self.out_data = pd.read_csv(self.file, sep=',', usecols=self.out_keys)
#                 self.out_data = pd2tensor(self.out_data, self.out_keys)
#
#     def __getitem__(self, index):
#         if self.mode == 'test':
#             return self.in_data[index]
#         else:
#             return self.in_data[index], self.out_data[index]
#
#     def __len__(self):
#         return self.in_data.size(0)



# -*- coding: utf-8 -*

import setuptools
from os import path
from io import open

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

about = {}
with open(path.join(here, 'mavis', '__info__.py'), encoding='utf-8') as f:
    exec(f.read(), about)

setuptools.setup(
    name='mix-mavis',
    version=about['__version__'],
    description='MAVIS 数据分析工具',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/mixlinker/mavis',
    author=about['__author__'],
    author_email=about['__author_email__'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    keywords='visualization, analysis, jupyter',
    packages=setuptools.find_packages(),
    python_requires='>=3.5',
    install_requires=[
        'IPython',
        'matplotlib',
        'seaborn',
        'ipywidgets',
        'numpy',
        'pandas',
        'scipy',
        'scikit-learn',
        'torch',
        'torchvision',
        'boto3'
    ])



# -*- coding: utf-8 -*-
###############################################################################
# Copyright (c) 2014-2019 Mixlinker Networks Inc. <mixiot@mixlinker.com>
# All rights reserved.
#
# This program and the accompanying materials are made available under the
# terms of the Application License of Mixlinker Networks License and Mixlinker
# Distribution License which accompany this distribution.
#
# The Mixlinker License is available at
#    http://www.mixlinker.com/legal/license.html
# and the Mixlinker Distribution License is available at
#    http://www.mixlinker.com/legal/distribution.html
#
# Contributors:
#    Mixlinker Technical Team
###############################################################################
# Description : Mixiot 数据查询
# Author : kanxiangyang

import ipywidgets as widgets
from ipywidgets import interact, interact_manual
from IPython.core.interactiveshell import InteractiveShell
from IPython.display import HTML, display, FileLink
import pandas as pd
import requests
import time
import json
import os
import base64
from ast import literal_eval
from datetime import datetime, timedelta
from . import config

interact_manual.opts['manual_name'] = '确认'
InteractiveShell.ast_node_interactivity = 'all'


def get_token(url, username, password):
    login_param = {'username': username, 'password': password, 'system': 'MixPro'}
    login_result = json.loads(requests.post(url=url, data=login_param).text)
    if login_result['code'] != 200:
        print(login_result['msg'])
        raise Exception
    if 'data' in list(login_result.keys()):
        token = login_result["data"]["token"]
    elif 'result' in list(login_result.keys()):
        token = login_result["result"]["token"]
    else:
        raise Exception
    return {'Authorization': 'Bearer ' + token}


def get_menu_list(url, headers):
    parameter = {"is_all": 1, "page_index": 1, "page_size": 20, "condition": ""}
    receive_data = json.loads(requests.post(url=url, headers=headers, data=parameter).text)
    if receive_data['code'] != 200:
        print("获取设备信息失败")
        raise Exception
    if 'data' in list(receive_data.keys()):
        result = receive_data['data']
    else:
        result = receive_data["result"]["data"]
    if not result:
        print("获取设备信息为空")
        raise Exception
    result = pd.DataFrame(result)
    result.index = result['equipment_id']
    result = result.drop(columns=['equipment_id']).sort_index(ascending=True)
    return result


def get_mapping(url, headers, equipment_id):
    parameter = {"equipment_id": equipment_id}
    receive_data = json.loads(requests.post(url=url, headers=headers, data=parameter).text)
    if receive_data['code'] != 200:
        print("获取mapping数据失败")
        raise Exception
    result = receive_data['result']['script']
    if not result:
        print("获取mapping数据为空")
        raise Exception
    result = pd.Series({i[0]: i[-1] for i in result})
    return result


def get_data(url, headers, equipment_id, start_time, end_time, page_size, keys):
    start_time = pd.to_datetime(start_time).strftime('%Y-%m-%d %H:%M:%S')
    end_time = pd.to_datetime(end_time).strftime('%Y-%m-%d %H:%M:%S')
    keys = str(list(keys)).replace("'", '"')
    parameter = {'equipment_id': equipment_id, 'page_size': page_size, 'page_index': '1',
                 'end_time': end_time, 'start_time': start_time, 'keys': keys}
    receive_data = json.loads(requests.post(url=url, headers=headers, data=parameter).text)
    if receive_data['code'] != 200:
        print("获取fidis数据失败")
    else:
        result = receive_data["result"]["data"]
        if not result:
            print("获取fidis数据为空")
        time_index = pd.to_datetime([j['time'] for j in result])
        df = pd.DataFrame([j['value'] for j in result], index=time_index).sort_index(ascending=True)
        return df


def export_data(url, headers, equipment_id, start_time, end_time, keys, h5, merge, resample, method):
    start_time = pd.to_datetime(start_time).strftime('%Y-%m-%d %H:%M:%S')
    end_time = pd.to_datetime(end_time).strftime('%Y-%m-%d %H:%M:%S')
    keys = str(list(keys))
    parameter = {'equipment_id': equipment_id, 'start_time': start_time, 'end_time': end_time, 'keys': keys,
                 'h5': h5, 'merge': merge, 'resample': resample, 'method': method}
    receive_data = json.loads(requests.post(url=url, headers=headers, data=parameter).text)
    if receive_data['code'] != 200:
        print("导出数据失败")
        print(receive_data)
        raise Exception
    else:
        status_id = receive_data['result']['state']
        status_url = url.split("/export/export_data")[0] + f"/status/{status_id}"
        download_url = url.split("/export/export_data")[0] + f"/download/{status_id}"
        print(f'请等待后台任务完成，当前任务ID为{status_id}')
        count = 0
        while True:
            time.sleep(10)
            receive_data = json.loads(requests.get(url=status_url).text)
            if receive_data['code'] != 200:
                break
            elif receive_data['result']['state'] == 'FAILURE':
                print("后台任务失败")
                break
            elif receive_data['result']['state'] == 'SUCCESS':
                print("后台任务成功")
                break
            else:
                count += 1
                print(f"任务进行中...{count * 10}秒")
    return HTML(f'<a href="{download_url}">下载数据</a>')


def create_download_link(df, title="Download CSV file", filename="data.csv"):
    csv = df.to_csv(encoding='utf_8_sig')
    b64 = base64.b64encode(csv.encode('utf_8_sig'))
    payload = b64.decode()
    html = '<a download="{filename}" href="data:text/csv;base64,{payload}" target="_blank">{title}</a>'
    html = html.format(payload=payload, title=title, filename=filename)
    return HTML(html)


class Mixiot(object):
    def __init__(self):
        self.platform_en = None
        self.equipment_id = None
        self.headers = None
        self.equipment_info = None
        self.mapping_info = None

    def live_plot(self, keys, interval):
        if 'status_code' in self.equipment_info.columns:
            if self.equipment_info.loc[self.equipment_id, 'status_code'] != 1:
                status_name = self.equipment_info.loc[self.equipment_id, 'status_name']
                print(f"警告: 设备当前状态为{status_name}")
        interval = literal_eval(interval)
        keys = [self.mapping_info[self.mapping_info == key].index[0] for key in keys]
        apix_url = config.API(platform=self.platform_en).apix_url()
        live_result = pd.DataFrame()
        while True:
            time_now = datetime.now()
            start_time = (time_now - timedelta(seconds=interval)).strftime('%Y-%m-%d %H:%M:%S')
            end_time = time_now.strftime('%Y-%m-%d %H:%M:%S')
            page_size = 10
            result = get_data(url=apix_url, headers=self.headers, equipment_id=self.equipment_id, start_time=start_time,
                              end_time=end_time, page_size=page_size, keys=keys)
            live_result.append(result)
            print(live_result)
            delta_seconds = (datetime.now() - time_now).seconds
            time.sleep(interval - delta_seconds)

    def select_mosaic(self, start_time, end_time, page_size, keys, cn=True, save=False):
        keys = [self.mapping_info[self.mapping_info == key].index[0] for key in keys]
        apix_url = config.API(platform=self.platform_en).apix_url()
        result = get_data(url=apix_url, headers=self.headers, equipment_id=self.equipment_id, start_time=start_time,
                          end_time=end_time, page_size=page_size, keys=keys)
        if 'status_code' in self.equipment_info.columns:
            if self.equipment_info.loc[self.equipment_id, 'status_code'] != 1:
                status_name = self.equipment_info.loc[self.equipment_id, 'status_name']
                print(f"警告: 设备当前状态为{status_name}")
        if cn:
            result.columns = self.mapping_info[result.columns]
        start_time = pd.to_datetime(start_time).strftime('%Y-%m-%d-%H-%M-%S')
        end_time = pd.to_datetime(end_time).strftime('%Y-%m-%d-%H-%M-%S')
        filename = f"{self.platform_en}_device{self.equipment_id}_{start_time}_{end_time}.csv"
        if not os.path.isdir("/data/tmp"):
            os.makedirs("/data/tmp", mode=0o777)
        result.to_csv(f"/data/tmp/{filename}", encoding='utf_8_sig')
        if save:
            display(FileLink(f"/data/tmp/{filename}"))
            return create_download_link(result, title="下载数据到本地", filename=filename)
        else:
            display(FileLink(f"/data/tmp/{filename}"))
            return result

    def select_device(self, equipment_name):
        self.equipment_id = self.equipment_info[self.equipment_info['equipment_name'] == equipment_name].index[0]
        apiq_pro_url = config.API(platform=self.platform_en).apiq_pro_url()
        self.mapping_info = get_mapping(url=apiq_pro_url, headers=self.headers, equipment_id=self.equipment_id)
        key_list = self.mapping_info.values.tolist()
        default_start_time = (datetime.now() - timedelta(days=1)).strftime('%Y-%m-%d %H:%M:%S')
        default_end_time = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        return interact_manual(self.select_mosaic,
                               keys=widgets.SelectMultiple(options=key_list,
                                                           value=key_list[:2],
                                                           description='设备参数'),
                               start_time=widgets.Text(value=default_start_time, description='开始时间', disabled=False),
                               end_time=widgets.Text(value=default_end_time, description='结束时间', disabled=False),
                               page_size=widgets.Text(value='100', description='请求条数', disabled=False),
                               cn=widgets.Checkbox(value=False, description='是否返回中文参数', disabled=False),
                               save=widgets.Checkbox(value=False, description='是否保存数据', disabled=False)
                               )

    def login(self, username, password):
        login_url = config.API(platform=self.platform_en).apiq_login_url()
        self.headers = get_token(url=login_url, username=username, password=password)
        apiq_info_url = config.API(platform=self.platform_en).apiq_info_url()
        self.equipment_info = get_menu_list(url=apiq_info_url, headers=self.headers)
        equipment_name_list = self.equipment_info['equipment_name'].values.tolist()
        return interact(self.select_device,
                        equipment_name=widgets.Dropdown(options=equipment_name_list,
                                                        value=equipment_name_list[0],
                                                        description='设备 '))

    def select_platform(self, platform_cn):
        self.platform_en = config.platform_repository[platform_cn]
        return interact_manual(self.login,
                               username=widgets.Text(value='admin', description='用户名', disabled=False),
                               password=widgets.Text(value='', description='密码', disabled=False)
                               )

    def __call__(self):
        return interact(self.select_platform,
                        platform_cn=widgets.Dropdown(options=list(config.platform_repository.keys()),
                                                     value=list(config.platform_repository.keys())[0],
                                                     description='MIXIOT 平台 '))


class Export(object):
    def __init__(self):
        self.platform_en = None
        self.equipment_id = None
        self.headers = None
        self.equipment_info = None
        self.mapping_info = None

    def select_mosaic(self, start_time, end_time, keys, h5, merge, resample, method):
        keys = [self.mapping_info[self.mapping_info == key].index[0] for key in keys]
        influx_export_url = config.API(platform=self.platform_en).influx_export_url()
        if resample and method:
            resample = config.resample_method[resample]
        else:
            resample = ""
            method = ""
        if 'status_code' in self.equipment_info.columns:
            if self.equipment_info.loc[self.equipment_id, 'status_code'] != 1:
                status_name = self.equipment_info.loc[self.equipment_id, 'status_name']
                print(f"警告: 设备当前状态为{status_name}")
        return export_data(url=influx_export_url, headers=self.headers, equipment_id=self.equipment_id,
                           start_time=start_time, end_time=end_time, keys=keys, h5=h5, merge=merge, resample=resample,
                           method=method)

    def select_device(self, equipment_name):
        self.equipment_id = self.equipment_info[self.equipment_info['equipment_name'] == equipment_name].index[0]
        apiq_pro_url = config.API(platform=self.platform_en).apiq_pro_url()
        self.mapping_info = get_mapping(url=apiq_pro_url, headers=self.headers, equipment_id=self.equipment_id)
        key_list = self.mapping_info.values.tolist()
        default_start_time = (datetime.now() - timedelta(days=1)).strftime('%Y-%m-%d %H:%M:%S')
        default_end_time = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        return interact_manual(self.select_mosaic,
                               keys=widgets.SelectMultiple(options=key_list,
                                                           value=key_list[:2],
                                                           description='设备参数'),
                               start_time=widgets.Text(value=default_start_time, description='开始时间', disabled=False),
                               end_time=widgets.Text(value=default_end_time, description='结束时间', disabled=False),
                               h5=widgets.Checkbox(value=False, description='是否保存为h5', disabled=False),
                               merge=widgets.Checkbox(value=True, description='是否合并', disabled=False),
                               resample=widgets.Text(value=None, description='重采样时间间隔/秒', disabled=False),
                               method=widgets.Dropdown(options=list(config.resample_method.keys()), value=None,
                                                       description='重采样方法', disabled=False)
                               )

    def login(self, username, password):
        login_url = config.API(platform=self.platform_en).apiq_login_url()
        self.headers = get_token(url=login_url, username=username, password=password)
        apiq_info_url = config.API(platform=self.platform_en).apiq_info_url()
        self.equipment_info = get_menu_list(url=apiq_info_url, headers=self.headers)
        equipment_name_list = self.equipment_info['equipment_name'].values.tolist()
        return interact(self.select_device,
                        equipment_name=widgets.Dropdown(options=equipment_name_list,
                                                        value=equipment_name_list[0],
                                                        description='设备 '))

    def select_platform(self, platform_cn):
        self.platform_en = config.platform_repository[platform_cn]
        return interact_manual(self.login,
                               username=widgets.Text(value='admin', description='用户名', disabled=False),
                               password=widgets.Text(value='', description='密码', disabled=False)
                               )

    def __call__(self):
        return interact(self.select_platform,
                        platform_cn=widgets.Dropdown(options=list(config.platform_repository.keys()),
                                                     value=list(config.platform_repository.keys())[0],
                                                     description='MIXIOT 平台 '))


class Mapping(object):
    def __init__(self):
        self.platform_en = None
        self.equipment_id = None
        self.headers = None
        self.equipment_info = None
        self.mapping_info = None

    def select_device(self, equipment_name, save):
        self.equipment_id = self.equipment_info[self.equipment_info['equipment_name'] == equipment_name].index[0]
        apiq_pro_url = config.API(platform=self.platform_en).apiq_pro_url()
        result = get_mapping(url=apiq_pro_url, headers=self.headers, equipment_id=self.equipment_id)
        result_df = pd.DataFrame(result, columns=['参数中文'])
        result_df.index.name = '参数ID'
        if save:
            filename = f"{self.platform_en}_{self.equipment_id}_mapping.csv"
            return create_download_link(result_df, title="下载数据到本地", filename=filename)
        else:
            return result_df

    def login(self, username, password):
        login_url = config.API(platform=self.platform_en).apiq_login_url()
        self.headers = get_token(url=login_url, username=username, password=password)
        apiq_info_url = config.API(platform=self.platform_en).apiq_info_url()
        self.equipment_info = get_menu_list(url=apiq_info_url, headers=self.headers)
        equipment_name_list = self.equipment_info['equipment_name'].values.tolist()
        return interact_manual(self.select_device,
                               equipment_name=widgets.Dropdown(options=equipment_name_list,
                                                               value=equipment_name_list[0],
                                                               description='设备 '),
                               save=widgets.Checkbox(value=False, description='是否保存', disabled=False))

    def select_platform(self, platform_cn):
        self.platform_en = config.platform_repository[platform_cn]
        return interact_manual(self.login,
                               username=widgets.Text(value='admin', description='用户名', disabled=False),
                               password=widgets.Text(value='', description='密码', disabled=False)
                               )

    def __call__(self):
        return interact(self.select_platform,
                        platform_cn=widgets.Dropdown(options=list(config.platform_repository.keys()),
                                                     value=list(config.platform_repository.keys())[0],
                                                     description='MIXIOT 平台 '))
